<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace TamerLib\Objects;

    use Exception;
    use LogLib\Log;
    use RuntimeException;
    use TamerLib\Classes\Utilities;

    class ServerConfiguration
    {
        /**
         * @var string
         */
        private $host;

        /**
         * @var int
         */
        private $port;

        /**
         * @var string|null
         */
        private $password;

        /**
         * @var int
         */
        private $database;

        /**
         * ServerConfiguration constructor.
         *
         * @param string $host
         * @param int|null $port
         * @param string|null $password
         * @param int $database
         */
        public function __construct(string $host='127.0.0.1', ?int $port=null, ?string $password=null, int $database=0)
        {
            $this->host = $host;
            $this->port = $port;
            $this->password = $password;
            $this->database = $database;

            if(is_null($port))
            {
                try
                {
                    $port = Utilities::getAvailablePort($host);
                }
                catch(Exception $e)
                {
                    Log::warning(Utilities::getName(), 'No available port found. Using random port.', $e);

                    try
                    {
                        $port = random_int(1024, 65535);
                    }
                    catch(Exception $e)
                    {
                        throw new RuntimeException('Could not generate random port.', 0, $e);
                    }
                }
                finally
                {
                    Log::debug(Utilities::getName(), 'Selected port ' . $port . '.');
                }
            }

            if(!is_null($port))
            {
                $this->port = $port;
            }
        }

        /**
         * @return string
         */
        public function getHost(): string
        {
            return $this->host;
        }

        /**
         * @return int
         */
        public function getPort(): int
        {
            return $this->port;
        }

        /**
         * @return string|null
         */
        public function getPassword(): ?string
        {
            return $this->password;
        }

        /**
         * @return int
         */
        public function getDatabase(): int
        {
            return $this->database;
        }
    }